@@ -41,8 +41,9 @@
 #include <deque>
 #include <vector>
 #include <ostream>
+#include <set>
+#include <memory>
 
-#include "SharedHandle.h"
 #include "File.h"
 #include "Request.h"
 #include "URIResult.h"
@@ -50,6 +51,7 @@
 #include "A2STR.h"
 #include "TimerA2.h"
 #include "util.h"
+#include "a2functional.h"
 
 namespace aria2 {
 
@@ -57,29 +59,44 @@ class URISelector;
 class ServerStatMan;
 
 class FileEntry {
+public:
+  typedef std::set<std::shared_ptr<Request>, RefLess<Request> >
+  InFlightRequestSet;
+
 private:
-  std::string path_;
+  class RequestFaster {
+  public:
+    bool operator()(const std::shared_ptr<Request>& lhs,
+                    const std::shared_ptr<Request>& rhs) const;
+  };
+  typedef std::set<std::shared_ptr<Request>, RequestFaster> RequestPool;
+
+  int64_t length_;
+  int64_t offset_;
+
   std::deque<std::string> uris_;
   std::deque<std::string> spentUris_;
-  uint64_t length_;
-  off_t offset_;
-  bool requested_;
-  std::deque<SharedHandle<Request> > requestPool_;
-  std::deque<SharedHandle<Request> > inFlightRequests_;
-  std::string contentType_;
   // URIResult is stored in the ascending order of the time when its result is
   // available.
   std::deque<URIResult> uriResults_;
-  bool uniqueProtocol_;
-  size_t maxConnectionPerServer_;
-  std::string originalName_;
-  Timer lastFasterReplace_;
+  RequestPool requestPool_;
+  InFlightRequestSet inFlightRequests_;
 
-  void storePool(const SharedHandle<Request>& request);
+  std::string path_;
+  std::string contentType_;
+  std::string originalName_;
+
+  Timer lastFasterReplace_;
+  int maxConnectionPerServer_;
+
+  bool requested_;
+  bool uniqueProtocol_;
+
+  void storePool(const std::shared_ptr<Request>& request);
 public:
   FileEntry();
 
-  FileEntry(const std::string& path, uint64_t length, off_t offset,
+  FileEntry(const std::string& path, int64_t length, int64_t offset,
             const std::vector<std::string>& uris = std::vector<std::string>());
 
   ~FileEntry();
@@ -92,17 +109,17 @@ public:
 
   const std::string& getPath() const { return path_; }
 
-  void setPath(const std::string& path);
+  void setPath(std::string path);
 
-  uint64_t getLength() const { return length_; }
+  int64_t getLength() const { return length_; }
 
-  void setLength(uint64_t length) { length_ = length; }
+  void setLength(int64_t length) { length_ = length; }
 
-  off_t getOffset() const { return offset_; }
+  int64_t getOffset() const { return offset_; }
 
-  void setOffset(off_t offset) { offset_ = offset; }
+  void setOffset(int64_t offset) { offset_ = offset; }
 
-  off_t getLastOffset() { return offset_+length_; }
+  int64_t getLastOffset() { return offset_+length_; }
 
   bool isRequested() const { return requested_; }
 
@@ -123,6 +140,12 @@ public:
     return spentUris_;
   }
 
+  // Exposed for unitest
+  std::deque<std::string>& getSpentUris()
+  {
+    return spentUris_;
+  }
+
   size_t setUris(const std::vector<std::string>& uris);
 
   template<typename InputIterator>
@@ -144,7 +167,7 @@ public:
   // Inserts uris_ and spentUris_ into uris.
   void getUris(std::vector<std::string>& uris) const;
 
-  void setContentType(const std::string& contentType);
+  void setContentType(std::string contentType);
 
   const std::string& getContentType() const { return contentType_; }
 
@@ -159,8 +182,8 @@ public:
   // returns Request object either because uris_ is empty or all URI
   // are not be usable because maxConnectionPerServer_ limit, then
   // reuse used URIs and do selection again.
-  SharedHandle<Request> getRequest
-  (const SharedHandle<URISelector>& selector,
+  std::shared_ptr<Request> getRequest
+  (URISelector* selector,
    bool uriReuse,
    const std::vector<std::pair<size_t, std::string> >& usedHosts,
    const std::string& referer = A2STR::NIL,
@@ -169,24 +192,24 @@ public:
   // Finds pooled Request object which is faster than passed one,
   // comparing their PeerStat objects. If such Request is found, it is
   // removed from the pool and returned.
-  SharedHandle<Request> findFasterRequest(const SharedHandle<Request>& base);
+  std::shared_ptr<Request> findFasterRequest(const std::shared_ptr<Request>& base);
 
   // Finds faster server using ServerStatMan.
-  SharedHandle<Request>
+  std::shared_ptr<Request>
   findFasterRequest
-  (const SharedHandle<Request>& base,
+  (const std::shared_ptr<Request>& base,
    const std::vector<std::pair<size_t, std::string> >& usedHosts,
-   const SharedHandle<ServerStatMan>& serverStatMan);
+   const std::shared_ptr<ServerStatMan>& serverStatMan);
 
-  void poolRequest(const SharedHandle<Request>& request);
+  void poolRequest(const std::shared_ptr<Request>& request);
 
-  bool removeRequest(const SharedHandle<Request>& request);
+  bool removeRequest(const std::shared_ptr<Request>& request);
 
   size_t countInFlightRequest() const;
 
   size_t countPooledRequest() const;
 
-  const std::deque<SharedHandle<Request> >& getInFlightRequests() const
+  const InFlightRequestSet& getInFlightRequests() const
   {
     return inFlightRequests_;
   }
@@ -196,7 +219,7 @@ public:
   bool exists() const;
 
   // Translate global offset goff to file local offset.
-  off_t gtoloff(off_t goff) const;
+  int64_t gtoloff(int64_t goff) const;
 
   void removeURIWhoseHostnameIs(const std::string& hostname);
 
@@ -214,12 +237,12 @@ public:
   void extractURIResult
   (std::deque<URIResult>& res, error_code::Value r);
 
-  void setMaxConnectionPerServer(size_t n)
+  void setMaxConnectionPerServer(int n)
   {
     maxConnectionPerServer_ = n;
   }
 
-  size_t getMaxConnectionPerServer() const
+  int getMaxConnectionPerServer() const
   {
     return maxConnectionPerServer_;
   }
@@ -231,7 +254,10 @@ public:
 
   void releaseRuntimeResource();
 
-  void setOriginalName(const std::string& originalName);
+  // Push URIs in pooled or in-flight requests to the front of uris_.
+  void putBackRequest();
+
+  void setOriginalName(std::string originalName);
 
   const std::string& getOriginalName() const
   {
@@ -255,9 +281,9 @@ public:
 
 // Returns the first FileEntry which isRequested() method returns
 // true.  If no such FileEntry exists, then returns
-// SharedHandle<FileEntry>().
+// std::shared_ptr<FileEntry>().
 template<typename InputIterator>
-SharedHandle<FileEntry> getFirstRequestedFileEntry
+std::shared_ptr<FileEntry> getFirstRequestedFileEntry
 (InputIterator first, InputIterator last)
 {
   for(; first != last; ++first) {
@@ -265,7 +291,7 @@ SharedHandle<FileEntry> getFirstRequestedFileEntry
       return *first;
     }
   }
-  return SharedHandle<FileEntry>();
+  return nullptr;
 }
 
 // Counts the number of files selected in the given iterator range
@@ -299,7 +325,7 @@ bool isUriSuppliedForRequsetFileEntry(InputIterator first, InputIterator last)
 // FileEntry, writes "n/a" to o.
 void writeFilePath
 (std::ostream& o,
- const SharedHandle<FileEntry>& entry,
+ const std::shared_ptr<FileEntry>& entry,
  bool memory);
 
 // Writes first filename to given o.  If memory is true, the output is
@@ -311,7 +337,7 @@ template<typename InputIterator>
 void writeFilePath
 (InputIterator first, InputIterator last, std::ostream& o, bool memory)
 {
-  SharedHandle<FileEntry> e = getFirstRequestedFileEntry(first, last);
+  std::shared_ptr<FileEntry> e = getFirstRequestedFileEntry(first, last);
   if(!e) {
     o << "n/a";
   } else {
