@@ -57,15 +57,25 @@
 
 namespace aria2 {
 
-MetalinkParserController::MetalinkParserController():
-  metalinker_(new Metalinker())
+MetalinkParserController::MetalinkParserController()
+  : metalinker_{make_unique<Metalinker>()}
 {}
 
 MetalinkParserController::~MetalinkParserController() {}
 
+void MetalinkParserController::reset()
+{
+  metalinker_ = make_unique<Metalinker>();
+}
+
+std::unique_ptr<Metalinker> MetalinkParserController::getResult()
+{
+  return std::move(metalinker_);
+}
+
 void MetalinkParserController::newEntryTransaction()
 {
-  tEntry_.reset(new MetalinkEntry());
+  tEntry_ = make_unique<MetalinkEntry>();
   tResource_.reset();
   tMetaurl_.reset();
 #ifdef ENABLE_MESSAGE_DIGEST
@@ -75,19 +85,19 @@ void MetalinkParserController::newEntryTransaction()
 #endif // ENABLE_MESSAGE_DIGEST
 }
 
-void MetalinkParserController::setFileNameOfEntry(const std::string& filename)
+void MetalinkParserController::setFileNameOfEntry(std::string filename)
 {
   if(!tEntry_) {
     return;
   }
   if(!tEntry_->file) {
-    tEntry_->file.reset(new FileEntry(util::escapePath(filename), 0, 0));
+    tEntry_->file = make_unique<FileEntry>(util::escapePath(filename), 0, 0);
   } else {
     tEntry_->file->setPath(util::escapePath(filename));
   }
 }
 
-void MetalinkParserController::setFileLengthOfEntry(uint64_t length)
+void MetalinkParserController::setFileLengthOfEntry(int64_t length)
 {
   if(!tEntry_) {
     return;
@@ -99,28 +109,28 @@ void MetalinkParserController::setFileLengthOfEntry(uint64_t length)
   tEntry_->sizeKnown = true;
 }
 
-void MetalinkParserController::setVersionOfEntry(const std::string& version)
+void MetalinkParserController::setVersionOfEntry(std::string version)
 {
   if(!tEntry_) {
     return;
   }
-  tEntry_->version = version;
+  tEntry_->version = std::move(version);
 }
 
-void MetalinkParserController::setLanguageOfEntry(const std::string& language)
+void MetalinkParserController::setLanguageOfEntry(std::string language)
 {
   if(!tEntry_) {
     return;
   }
-  tEntry_->languages.push_back(language);
+  tEntry_->languages.push_back(std::move(language));
 }
 
-void MetalinkParserController::setOSOfEntry(const std::string& os)
+void MetalinkParserController::setOSOfEntry(std::string os)
 {
   if(!tEntry_) {
     return;
   }
-  tEntry_->oses.push_back(os);
+  tEntry_->oses.push_back(std::move(os));
 }
 
 void MetalinkParserController::setMaxConnectionsOfEntry(int maxConnections)
@@ -142,8 +152,7 @@ void MetalinkParserController::commitEntryTransaction()
   commitChunkChecksumTransactionV4();
   commitChunkChecksumTransaction();
   commitSignatureTransaction();
-  metalinker_->addEntry(tEntry_);
-  tEntry_.reset();
+  metalinker_->addEntry(std::move(tEntry_));
 }
 
 void MetalinkParserController::cancelEntryTransaction()
@@ -162,52 +171,52 @@ void MetalinkParserController::newResourceTransaction()
   if(!tEntry_) {
     return;
   }
-  tResource_.reset(new MetalinkResource());
+  tResource_ = make_unique<MetalinkResource>();
 }
 
-void MetalinkParserController::setURLOfResource(const std::string& url)
+void MetalinkParserController::setURLOfResource(std::string url)
 {
   if(!tResource_) {
     return;
   }
   std::string u = uri::joinUri(baseUri_, url);
-  uri::UriStruct us;
-  if(uri::parse(us, u)) {
-    tResource_->url = u;
+  uri_split_result us;
+  if(uri_split(&us, u.c_str()) == 0) {
+    tResource_->url = std::move(u);
     if(tResource_->type == MetalinkResource::TYPE_UNKNOWN) {
-      setTypeOfResource(us.protocol);
+      setTypeOfResource(uri::getFieldString(us, USR_SCHEME,
+                                            tResource_->url.c_str()));
     }
   } else {
-    tResource_->url = url;
+    tResource_->url = std::move(url);
   }
 }
 
-void MetalinkParserController::setTypeOfResource(const std::string& type)
+void MetalinkParserController::setTypeOfResource(std::string type)
 {
   if(!tResource_) {
     return;
   }
-  if(type == MetalinkResource::FTP) {
+  if(type == "ftp") {
     tResource_->type = MetalinkResource::TYPE_FTP;
-  } else if(type == MetalinkResource::HTTP) {
+  } else if(type == "http") {
     tResource_->type = MetalinkResource::TYPE_HTTP;
-  } else if(type == MetalinkResource::HTTPS) {
+  } else if(type == "https") {
     tResource_->type = MetalinkResource::TYPE_HTTPS;
-  } else if(type == MetalinkResource::BITTORRENT) {
-    tResource_->type = MetalinkResource::TYPE_BITTORRENT;
-  } else if(type == MetalinkResource::TORRENT) { // Metalink4Spec
+  } else if(type == "bittorrent" || type == "torrent") {
+    // "torrent" is Metalink4Spec
     tResource_->type = MetalinkResource::TYPE_BITTORRENT;
   } else {
     tResource_->type = MetalinkResource::TYPE_NOT_SUPPORTED;
   }
 }
 
-void MetalinkParserController::setLocationOfResource(const std::string& location)
+void MetalinkParserController::setLocationOfResource(std::string location)
 {
   if(!tResource_) {
     return;
   }
-  tResource_->location = location;
+  tResource_->location = std::move(location);
 }
 
 void MetalinkParserController::setPriorityOfResource(int priority)
@@ -233,16 +242,16 @@ void MetalinkParserController::commitResourceTransaction()
   }
 #ifdef ENABLE_BITTORRENT
   if(tResource_->type == MetalinkResource::TYPE_BITTORRENT) {
-    SharedHandle<MetalinkMetaurl> metaurl(new MetalinkMetaurl());
-    metaurl->url = tResource_->url;
+    auto metaurl = make_unique<MetalinkMetaurl>();
+    metaurl->url = std::move(tResource_->url);
     metaurl->priority = tResource_->priority;
     metaurl->mediatype = MetalinkMetaurl::MEDIATYPE_TORRENT;
-    tEntry_->metaurls.push_back(metaurl);
+    tEntry_->metaurls.push_back(std::move(metaurl));
   } else {
-    tEntry_->resources.push_back(tResource_);
+    tEntry_->resources.push_back(std::move(tResource_));
   }
 #else // !ENABLE_BITTORRENT
-  tEntry_->resources.push_back(tResource_);
+  tEntry_->resources.push_back(std::move(tResource_));
 #endif // !ENABLE_BITTORRENT
   tResource_.reset();
 }
@@ -258,11 +267,11 @@ void MetalinkParserController::newChecksumTransaction()
   if(!tEntry_) {
     return;
   }
-  tChecksum_.reset(new Checksum());
+  tChecksum_ = make_unique<Checksum>();
 #endif // ENABLE_MESSAGE_DIGEST
 }
 
-void MetalinkParserController::setTypeOfChecksum(const std::string& type)
+void MetalinkParserController::setTypeOfChecksum(std::string type)
 {
 #ifdef ENABLE_MESSAGE_DIGEST
   if(!tChecksum_) {
@@ -270,21 +279,21 @@ void MetalinkParserController::setTypeOfChecksum(const std::string& type)
   }
   std::string calgo = MessageDigest::getCanonicalHashType(type);
   if(MessageDigest::supports(calgo)) {
-    tChecksum_->setHashType(calgo);
+    tChecksum_->setHashType(std::move(calgo));
   } else {
     cancelChecksumTransaction();
   }
 #endif // ENABLE_MESSAGE_DIGEST
 }
 
-void MetalinkParserController::setHashOfChecksum(const std::string& md)
+void MetalinkParserController::setHashOfChecksum(std::string md)
 {
 #ifdef ENABLE_MESSAGE_DIGEST
   if(!tChecksum_) {
     return;
   }
   if(MessageDigest::isValidHash(tChecksum_->getHashType(), md)) {
-    tChecksum_->setDigest(md);
+    tChecksum_->setDigest(util::fromHex(md.begin(), md.end()));
   } else {
     cancelChecksumTransaction();
   }
@@ -300,7 +309,7 @@ void MetalinkParserController::commitChecksumTransaction()
   if(!tEntry_->checksum ||
      MessageDigest::isStronger(tChecksum_->getHashType(),
                                tEntry_->checksum->getHashType())) {
-    tEntry_->checksum = tChecksum_;
+    tEntry_->checksum = std::move(tChecksum_);
   }
   tChecksum_.reset();
 #endif // ENABLE_MESSAGE_DIGEST
@@ -319,12 +328,12 @@ void MetalinkParserController::newChunkChecksumTransactionV4()
   if(!tEntry_) {
     return;
   }
-  tChunkChecksumV4_.reset(new ChunkChecksum());
+  tChunkChecksumV4_ = make_unique<ChunkChecksum>();
   tempChunkChecksumsV4_.clear();
 #endif // ENABLE_MESSAGE_DIGEST
 }
 
-void MetalinkParserController::setTypeOfChunkChecksumV4(const std::string& type)
+void MetalinkParserController::setTypeOfChunkChecksumV4(std::string type)
 {
 #ifdef ENABLE_MESSAGE_DIGEST
   if(!tChunkChecksumV4_) {
@@ -332,7 +341,7 @@ void MetalinkParserController::setTypeOfChunkChecksumV4(const std::string& type)
   }
   std::string calgo = MessageDigest::getCanonicalHashType(type);
   if(MessageDigest::supports(calgo)) {
-    tChunkChecksumV4_->setHashType(calgo);
+    tChunkChecksumV4_->setHashType(std::move(calgo));
   } else {
     cancelChunkChecksumTransactionV4();
   }
@@ -353,18 +362,18 @@ void MetalinkParserController::setLengthOfChunkChecksumV4(size_t length)
 #endif // ENABLE_MESSAGE_DIGEST
 }
 
-void MetalinkParserController::addHashOfChunkChecksumV4(const std::string& md)
+void MetalinkParserController::addHashOfChunkChecksumV4(std::string md)
 {
 #ifdef ENABLE_MESSAGE_DIGEST
   if(!tChunkChecksumV4_) {
     return;
   }
   if(MessageDigest::isValidHash(tChunkChecksumV4_->getHashType(), md)) {
-    tempChunkChecksumsV4_.push_back(md);
+    tempChunkChecksumsV4_.push_back(util::fromHex(md.begin(), md.end()));
   } else {
     cancelChunkChecksumTransactionV4();
   }
-#endif // ENABLE_MESSAGE_DIGEST  
+#endif // ENABLE_MESSAGE_DIGEST
 }
 
 void MetalinkParserController::commitChunkChecksumTransactionV4()
@@ -376,10 +385,8 @@ void MetalinkParserController::commitChunkChecksumTransactionV4()
   if(!tEntry_->chunkChecksum ||
      MessageDigest::isStronger(tChunkChecksumV4_->getHashType(),
                                tEntry_->chunkChecksum->getHashType())) {
-    std::vector<std::string> pieceHashes(tempChunkChecksumsV4_.begin(),
-                                         tempChunkChecksumsV4_.end());
-    tChunkChecksumV4_->setPieceHashes(pieceHashes);
-    tEntry_->chunkChecksum = tChunkChecksumV4_;
+    tChunkChecksumV4_->setPieceHashes(std::move(tempChunkChecksumsV4_));
+    tEntry_->chunkChecksum = std::move(tChunkChecksumV4_);
   }
   tChunkChecksumV4_.reset();
 #endif // ENABLE_MESSAGE_DIGEST
@@ -398,12 +405,12 @@ void MetalinkParserController::newChunkChecksumTransaction()
   if(!tEntry_) {
     return;
   }
-  tChunkChecksum_.reset(new ChunkChecksum());
+  tChunkChecksum_ = make_unique<ChunkChecksum>();
   tempChunkChecksums_.clear();
 #endif // ENABLE_MESSAGE_DIGEST
 }
 
-void MetalinkParserController::setTypeOfChunkChecksum(const std::string& type)
+void MetalinkParserController::setTypeOfChunkChecksum(std::string type)
 {
 #ifdef ENABLE_MESSAGE_DIGEST
   if(!tChunkChecksum_) {
@@ -411,7 +418,7 @@ void MetalinkParserController::setTypeOfChunkChecksum(const std::string& type)
   }
   std::string calgo = MessageDigest::getCanonicalHashType(type);
   if(MessageDigest::supports(calgo)) {
-    tChunkChecksum_->setHashType(calgo);
+    tChunkChecksum_->setHashType(std::move(calgo));
   } else {
     cancelChunkChecksumTransaction();
   }
@@ -432,14 +439,15 @@ void MetalinkParserController::setLengthOfChunkChecksum(size_t length)
 #endif // ENABLE_MESSAGE_DIGEST
 }
 
-void MetalinkParserController::addHashOfChunkChecksum(size_t order, const std::string& md)
+void MetalinkParserController::addHashOfChunkChecksum(size_t order,
+                                                      std::string md)
 {
 #ifdef ENABLE_MESSAGE_DIGEST
   if(!tChunkChecksum_) {
     return;
   }
   if(MessageDigest::isValidHash(tChunkChecksum_->getHashType(), md)) {
-    tempChunkChecksums_.push_back(std::make_pair(order, md));
+    tempChunkChecksums_.push_back(std::make_pair(order, std::move(md)));
   } else {
     cancelChunkChecksumTransaction();
   }
@@ -456,14 +464,14 @@ void MetalinkParserController::createNewHashOfChunkChecksum(size_t order)
 #endif // ENABLE_MESSAGE_DIGEST
 }
 
-void MetalinkParserController::setMessageDigestOfChunkChecksum(const std::string& md)
+void MetalinkParserController::setMessageDigestOfChunkChecksum(std::string md)
 {
 #ifdef ENABLE_MESSAGE_DIGEST
   if(!tChunkChecksum_) {
     return;
   }
   if(MessageDigest::isValidHash(tChunkChecksum_->getHashType(), md)) {
-    tempHashPair_.second = md;
+    tempHashPair_.second = util::fromHex(md.begin(), md.end());
   } else {
     cancelChunkChecksumTransaction();
   }
@@ -489,14 +497,15 @@ void MetalinkParserController::commitChunkChecksumTransaction()
   if(!tEntry_->chunkChecksum ||
      MessageDigest::isStronger(tChunkChecksum_->getHashType(),
                                tEntry_->chunkChecksum->getHashType())) {
-    std::sort(tempChunkChecksums_.begin(), tempChunkChecksums_.end(),
-              Ascend1st<std::pair<size_t, std::string> >());
+    std::sort(std::begin(tempChunkChecksums_), std::end(tempChunkChecksums_));
     std::vector<std::string> pieceHashes;
-    std::transform(tempChunkChecksums_.begin(), tempChunkChecksums_.end(),
+    std::transform(std::begin(tempChunkChecksums_),
+                   std::end(tempChunkChecksums_),
                    std::back_inserter(pieceHashes),
-                   select2nd<std::pair<size_t, std::string> >());
-    tChunkChecksum_->setPieceHashes(pieceHashes);
-    tEntry_->chunkChecksum = tChunkChecksum_;
+                   [](const std::pair<size_t, std::string>& p)
+                   { return p.second; });
+    tChunkChecksum_->setPieceHashes(std::move(pieceHashes));
+    tEntry_->chunkChecksum = std::move(tChunkChecksum_);
   }
   tChunkChecksum_.reset();
 #endif // ENABLE_MESSAGE_DIGEST
@@ -514,31 +523,31 @@ void MetalinkParserController::newSignatureTransaction()
   if(!tEntry_) {
     return;
   }
-  tSignature_.reset(new Signature());
+  tSignature_ = make_unique<Signature>();
 }
 
-void MetalinkParserController::setTypeOfSignature(const std::string& type)
+void MetalinkParserController::setTypeOfSignature(std::string type)
 {
   if(!tSignature_) {
     return;
   }
-  tSignature_->setType(type);
+  tSignature_->setType(std::move(type));
 }
 
-void MetalinkParserController::setFileOfSignature(const std::string& file)
+void MetalinkParserController::setFileOfSignature(std::string file)
 {
   if(!tSignature_) {
     return;
   }
-  tSignature_->setFile(file);
+  tSignature_->setFile(std::move(file));
 }
 
-void MetalinkParserController::setBodyOfSignature(const std::string& body)
+void MetalinkParserController::setBodyOfSignature(std::string body)
 {
   if(!tSignature_) {
     return;
   }
-  tSignature_->setBody(body);
+  tSignature_->setBody(std::move(body));
 }
 
 void MetalinkParserController::commitSignatureTransaction()
@@ -546,8 +555,7 @@ void MetalinkParserController::commitSignatureTransaction()
   if(!tSignature_) {
     return;
   }
-  tEntry_->setSignature(tSignature_);
-  tSignature_.reset();
+  tEntry_->setSignature(std::move(tSignature_));
 }
 
 void MetalinkParserController::cancelSignatureTransaction()
@@ -560,37 +568,35 @@ void MetalinkParserController::newMetaurlTransaction()
   if(!tEntry_) {
     return;
   }
-  tMetaurl_.reset(new MetalinkMetaurl());
+  tMetaurl_ = make_unique<MetalinkMetaurl>();
 }
 
-void MetalinkParserController::setURLOfMetaurl(const std::string& url)
+void MetalinkParserController::setURLOfMetaurl(std::string url)
 {
   if(!tMetaurl_) {
     return;
   }
 #ifdef ENABLE_BITTORRENT
   if(magnet::parse(url)) {
-    tMetaurl_->url = url;
+    tMetaurl_->url = std::move(url);
   } else
 #endif // ENABLE_BITTORRENT
     {
       std::string u = uri::joinUri(baseUri_, url);
-      uri::UriStruct us;
-      if(uri::parse(us, u)) {
-        tMetaurl_->url = u;
+      if(uri_split(nullptr, u.c_str()) == 0) {
+        tMetaurl_->url = std::move(u);
       } else {
-        tMetaurl_->url = url;
+        tMetaurl_->url = std::move(url);
       }
     }
 }
 
-void MetalinkParserController::setMediatypeOfMetaurl
-(const std::string& mediatype)
+void MetalinkParserController::setMediatypeOfMetaurl(std::string mediatype)
 {
   if(!tMetaurl_) {
     return;
   }
-  tMetaurl_->mediatype = mediatype;
+  tMetaurl_->mediatype = std::move(mediatype);
 }
 
 void MetalinkParserController::setPriorityOfMetaurl(int priority)
@@ -601,12 +607,12 @@ void MetalinkParserController::setPriorityOfMetaurl(int priority)
   tMetaurl_->priority = priority;
 }
 
-void MetalinkParserController::setNameOfMetaurl(const std::string& name)
+void MetalinkParserController::setNameOfMetaurl(std::string name)
 {
   if(!tMetaurl_) {
     return;
   }
-  tMetaurl_->name = name;
+  tMetaurl_->name = std::move(name);
 }
 
 void MetalinkParserController::commitMetaurlTransaction()
@@ -616,7 +622,7 @@ void MetalinkParserController::commitMetaurlTransaction()
   }
 #ifdef ENABLE_BITTORRENT
   if(tMetaurl_->mediatype == MetalinkMetaurl::MEDIATYPE_TORRENT) {
-    tEntry_->metaurls.push_back(tMetaurl_);
+    tEntry_->metaurls.push_back(std::move(tMetaurl_));
   }
 #endif // ENABLE_BITTORRENT
   tMetaurl_.reset();
