@@ -43,64 +43,58 @@
 #include "A2STR.h"
 #include "util.h"
 #include "BufferedFile.h"
+#include "array_fun.h"
 
 namespace aria2 {
 
 Authenticator::Authenticator() {}
 
 Authenticator::Authenticator
-(const std::string& machine,
- const std::string& login,
- const std::string& password,
- const std::string& account)
-  : machine_(machine),
-    login_(login),
-    password_(password),
-    account_(account)
+(std::string machine,
+ std::string login,
+ std::string password,
+ std::string account)
+  : machine_(std::move(machine)),
+    login_(std::move(login)),
+    password_(std::move(password)),
+    account_(std::move(account))
 {}
 
 Authenticator::~Authenticator() {}
 
 bool Authenticator::match(const std::string& hostname) const
 {
-  if(util::isNumericHost(hostname)) {
-    return hostname == machine_;
-  } else {
-    if(util::startsWith(machine_, A2STR::DOT_C)) {
-      return util::endsWith(A2STR::DOT_C+hostname, machine_);
-    } else {
-      return hostname == machine_;
-    }
-  }
+  return util::noProxyDomainMatch(hostname, machine_);
 }
 
-void Authenticator::setMachine(const std::string& machine)
+void Authenticator::setMachine(std::string machine)
 {
-  machine_ = machine;
+  machine_ = std::move(machine);
 }
 
-void Authenticator::setLogin(const std::string& login)
+void Authenticator::setLogin(std::string login)
 {
-  login_ = login;
+  login_ = std::move(login);
 }
 
-void Authenticator::setPassword(const std::string& password)
+void Authenticator::setPassword(std::string password)
 {
-  password_ = password;
+  password_ = std::move(password);
 }
 
-void Authenticator::setAccount(const std::string& account)
+void Authenticator::setAccount(std::string account)
 {
-  account_ = account;
+  account_ = std::move(account);
 }
 
 DefaultAuthenticator::DefaultAuthenticator() {}
 
 DefaultAuthenticator::DefaultAuthenticator
-(const std::string& login,
- const std::string& password,
- const std::string& account)
-  : Authenticator(A2STR::NIL, login, password, account)
+(std::string login,
+ std::string password,
+ std::string account)
+  : Authenticator("", std::move(login), std::move(password),
+                  std::move(account))
 {}
 
 DefaultAuthenticator::~DefaultAuthenticator() {}
@@ -114,20 +108,24 @@ Netrc::Netrc() {}
 
 Netrc::~Netrc() {}
 
-void Netrc::addAuthenticator(const SharedHandle<Authenticator>& authenticator)
+void Netrc::addAuthenticator(std::unique_ptr<Authenticator> authenticator)
 {
-  authenticators_.push_back(authenticator);
+  authenticators_.push_back(std::move(authenticator));
 }
 
 namespace {
 void skipMacdef(BufferedFile& fp)
 {
-  char buf[4096];
+  std::string s;
   while(1) {
-    if(!fp.gets(buf, sizeof(buf))) {
+    s = fp.getLine();
+    if(s.empty() || fp.eof()) {
       break;
     }
-    if(buf[0] == '\n' || buf[0] == '\r') {
+    if(!fp) {
+      throw DL_ABORT_EX("Netrc:I/O error.");
+    }
+    if(s[0] == '\n' || s[0] == '\r') {
       break;
     }
   }
@@ -137,7 +135,7 @@ void skipMacdef(BufferedFile& fp)
 void Netrc::parse(const std::string& path)
 {
   authenticators_.clear();
-  BufferedFile fp(path, BufferedFile::READ);
+  BufferedFile fp(path.c_str(), BufferedFile::READ);
   if(!fp) {
     throw DL_ABORT_EX(fmt("Cannot open file: %s", path.c_str()));
   }
@@ -149,73 +147,79 @@ void Netrc::parse(const std::string& path)
     SET_ACCOUNT,
     SET_MACDEF
   };
-  SharedHandle<Authenticator> authenticator;
+  std::unique_ptr<Authenticator> authenticator;
   STATE state = GET_TOKEN;
-  char buf[4096];
   while(1) {
-    if(!fp.getsn(buf, sizeof(buf))) {
-      break;
+    std::string line = fp.getLine();
+    if(line.empty()) {
+      if(fp.eof()) {
+        break;
+      } else if(!fp) {
+        throw DL_ABORT_EX("Netrc:I/O error.");
+      } else {
+        continue;
+      }
     }
-    std::string line(buf);
-    if(util::startsWith(line, "#")) {
+    if(line[0] == '#') {
       continue;
     }
-    std::vector<std::string> tokens;
-    util::split(line, std::back_inserter(tokens), " \t", true);
-    for(std::vector<std::string>::const_iterator iter = tokens.begin(),
+    std::vector<Scip> tokens;
+    util::splitIterM(line.begin(), line.end(), std::back_inserter(tokens),
+                     " \t", true);
+    for(std::vector<Scip>::const_iterator iter = tokens.begin(),
           eoi = tokens.end(); iter != eoi; ++iter) {
-      const std::string& token = *iter;
       if(state == GET_TOKEN) {
-        if(token == "machine") {
-          storeAuthenticator(authenticator);
+        if(util::streq((*iter).first, (*iter).second, "machine")) {
+          storeAuthenticator(std::move(authenticator));
           authenticator.reset(new Authenticator());
           state = SET_MACHINE;
-        } else if(token == "default") {
-          storeAuthenticator(authenticator);
+        } else if(util::streq((*iter).first, (*iter).second, "default")) {
+          storeAuthenticator(std::move(authenticator));
           authenticator.reset(new DefaultAuthenticator());
         } else {
           if(!authenticator) {
             throw DL_ABORT_EX
               (fmt("Netrc:parse error. %s encounterd where 'machine'"
-                   " or 'default' expected.", token.c_str()));
+                   " or 'default' expected.",
+                   std::string((*iter).first, (*iter).second).c_str()));
           }
-          if(token == "login") {
+          if(util::streq((*iter).first, (*iter).second, "login")) {
             state = SET_LOGIN;
-          } else if(token == "password") {
+          } else if(util::streq((*iter).first, (*iter).second, "password")) {
             state = SET_PASSWORD;
-          } else if(token == "account") {
+          } else if(util::streq((*iter).first, (*iter).second, "account")) {
             state = SET_ACCOUNT;
-          } else if(token == "macdef") {
+          } else if(util::streq((*iter).first, (*iter).second, "macdef")) {
             state = SET_MACDEF;
           }
         }
       } else {
         if(state == SET_MACHINE) {
-          authenticator->setMachine(token);
+          authenticator->setMachine({(*iter).first, (*iter).second});
         } else if(state == SET_LOGIN) {
-          authenticator->setLogin(token);
+          authenticator->setLogin({(*iter).first, (*iter).second});
         } else if(state == SET_PASSWORD) {
-          authenticator->setPassword(token);
+          authenticator->setPassword({(*iter).first, (*iter).second});
         } else if(state == SET_ACCOUNT) {
-          authenticator->setAccount(token);
+          authenticator->setAccount({(*iter).first, (*iter).second});
         } else if(state == SET_MACDEF) {
           skipMacdef(fp);
         }
         state = GET_TOKEN;
-      } 
+      }
     }
   }
   if(state != GET_TOKEN) {
     throw DL_ABORT_EX
       ("Netrc:parse error. EOF reached where a token expected.");
   }
-  storeAuthenticator(authenticator);
+  storeAuthenticator(std::move(authenticator));
 }
 
-void Netrc::storeAuthenticator(const SharedHandle<Authenticator>& authenticator)
+void Netrc::storeAuthenticator(std::unique_ptr<Authenticator> authenticator)
 {
   if(authenticator) {
-    authenticators_.push_back(authenticator);
+    authenticators_.push_back(std::move(authenticator));
   }
 }
 
@@ -226,24 +230,31 @@ private:
 public:
   AuthHostMatch(const std::string& hostname):hostname(hostname) {}
 
-  bool operator()(const SharedHandle<Authenticator>& authenticator)
+  bool operator()(const std::unique_ptr<Authenticator>& authenticator)
   {
     return authenticator->match(hostname);
   }
 };
 } // namespace
 
-SharedHandle<Authenticator>
+const Authenticator*
 Netrc::findAuthenticator(const std::string& hostname) const
 {
-  SharedHandle<Authenticator> res;
-  std::vector<SharedHandle<Authenticator> >::const_iterator itr =
-    std::find_if(authenticators_.begin(), authenticators_.end(),
-                 AuthHostMatch(hostname));
-  if(itr != authenticators_.end()) {
-    res = *itr;
+  std::unique_ptr<Authenticator> res;
+  auto itr = std::find_if(std::begin(authenticators_),
+                          std::end(authenticators_),
+                          AuthHostMatch(hostname));
+  if(itr == std::end(authenticators_)) {
+    return nullptr;
+  } else {
+    return (*itr).get();
   }
-  return res;
+}
+
+const std::vector<std::unique_ptr<Authenticator>>&
+Netrc::getAuthenticators() const
+{
+  return authenticators_;
 }
 
 } // namespace aria2
