@@ -37,38 +37,35 @@
 #include "DownloadEngine.h"
 #include "Option.h"
 #include "DlAbortEx.h"
-#include "Socket.h"
+#include "SocketCore.h"
 #include "Logger.h"
+#include "LogFactory.h"
 #include "message.h"
 #include "prefs.h"
 #include "DownloadFailureException.h"
-#include "StringFormat.h"
+#include "fmt.h"
 #include "wallclock.h"
 #include "util.h"
-#include "RequestGroupMan.h"
-#include "FileAllocationEntry.h"
-#include "CheckIntegrityEntry.h"
-#include "ServerStatMan.h"
-#include "FileEntry.h"
 
 namespace aria2 {
 
-PeerAbstractCommand::PeerAbstractCommand(cuid_t cuid,
-                                         const SharedHandle<Peer>& peer,
-                                         DownloadEngine* e,
-                                         const SocketHandle& s):
-  Command(cuid),
-  checkPoint_(global::wallclock),
-  // TODO referring global option
-  timeout_(e->getOption()->getAsInt(PREF_BT_TIMEOUT)),
-  e_(e),
-  socket_(s),
-  peer_(peer),
-  checkSocketIsReadable_(false),
-  checkSocketIsWritable_(false),
-  noCheck_(false)
+PeerAbstractCommand::PeerAbstractCommand
+(cuid_t cuid,
+ const std::shared_ptr<Peer>& peer,
+ DownloadEngine* e,
+ const std::shared_ptr<SocketCore>& s)
+ : Command(cuid),
+   checkPoint_(global::wallclock()),
+   // TODO referring global option
+   timeout_(e->getOption()->getAsInt(PREF_BT_TIMEOUT)),
+   e_(e),
+   socket_(s),
+   peer_(peer),
+   checkSocketIsReadable_(false),
+   checkSocketIsWritable_(false),
+   noCheck_(false)
 {
-  if(!socket_.isNull() && socket_->isOpen()) {
+  if(socket_ && socket_->isOpen()) {
     setReadCheckSocket(socket_);
   }
 }
@@ -81,14 +78,12 @@ PeerAbstractCommand::~PeerAbstractCommand()
 
 bool PeerAbstractCommand::execute()
 {
-  if(getLogger()->debug()) {
-    getLogger()->debug("CUID#%s -"
-                       " socket: read:%d, write:%d, hup:%d, err:%d, noCheck:%d",
-                       util::itos(getCuid()).c_str(),
-                       readEventEnabled(), writeEventEnabled(),
-                       hupEventEnabled(), errorEventEnabled(),
-                       noCheck_);
-  }
+  A2_LOG_DEBUG(fmt("CUID#%" PRId64 " -"
+                   " socket: read:%d, write:%d, hup:%d, err:%d, noCheck:%d",
+                   getCuid(),
+                   readEventEnabled(), writeEventEnabled(),
+                   hupEventEnabled(), errorEventEnabled(),
+                   noCheck_));
   if(exitBeforeExecute()) {
     onAbort();
     return true;
@@ -98,30 +93,29 @@ bool PeerAbstractCommand::execute()
        (checkSocketIsReadable_ && readEventEnabled()) ||
        (checkSocketIsWritable_ && writeEventEnabled()) ||
        hupEventEnabled()) {
-      checkPoint_ = global::wallclock;
+      checkPoint_ = global::wallclock();
     } else if(errorEventEnabled()) {
       throw DL_ABORT_EX
-        (StringFormat(MSG_NETWORK_PROBLEM,
-                      socket_->getSocketError().c_str()).str());
+        (fmt(MSG_NETWORK_PROBLEM,
+             socket_->getSocketError().c_str()));
     }
-    if(checkPoint_.difference(global::wallclock) >= timeout_) {
+    if(checkPoint_.difference(global::wallclock()) >= timeout_) {
       throw DL_ABORT_EX(EX_TIME_OUT);
     }
     return executeInternal();
   } catch(DownloadFailureException& err) {
-    getLogger()->error(EX_DOWNLOAD_ABORTED, err);
+    A2_LOG_ERROR_EX(EX_DOWNLOAD_ABORTED, err);
     onAbort();
-    onFailure();
+    onFailure(err);
     return true;
   } catch(RecoverableException& err) {
-    if(getLogger()->debug()) {
-      getLogger()->debug(MSG_TORRENT_DOWNLOAD_ABORTED, err,
-                         util::itos(getCuid()).c_str());
-      getLogger()->debug(MSG_PEER_BANNED,
-                         util::itos(getCuid()).c_str(),
-                         peer_->getIPAddress().c_str(),
-                         peer_->getPort());
-    }
+    A2_LOG_DEBUG_EX(fmt(MSG_TORRENT_DOWNLOAD_ABORTED,
+                        getCuid()),
+                    err);
+    A2_LOG_DEBUG(fmt(MSG_PEER_BANNED,
+                     getCuid(),
+                     peer_->getIPAddress().c_str(),
+                     peer_->getPort()));
     onAbort();
     return prepareForNextPeer(0);
   }
@@ -139,16 +133,16 @@ void PeerAbstractCommand::disableReadCheckSocket()
     e_->deleteSocketForReadCheck(readCheckTarget_, this);
     checkSocketIsReadable_ = false;
     readCheckTarget_.reset();
-  }  
+  }
 }
 
-void PeerAbstractCommand::setReadCheckSocket(const SocketHandle& socket)
+void PeerAbstractCommand::setReadCheckSocket(const std::shared_ptr<SocketCore>& socket)
 {
   if(!socket->isOpen()) {
     disableReadCheckSocket();
   } else {
     if(checkSocketIsReadable_) {
-      if(readCheckTarget_ != socket) {
+      if(*readCheckTarget_ != *socket) {
         e_->deleteSocketForReadCheck(readCheckTarget_, this);
         e_->addSocketForReadCheck(socket, this);
         readCheckTarget_ = socket;
@@ -170,13 +164,13 @@ void PeerAbstractCommand::disableWriteCheckSocket()
   }
 }
 
-void PeerAbstractCommand::setWriteCheckSocket(const SocketHandle& socket)
+void PeerAbstractCommand::setWriteCheckSocket(const std::shared_ptr<SocketCore>& socket)
 {
   if(!socket->isOpen()) {
     disableWriteCheckSocket();
   } else {
     if(checkSocketIsWritable_) {
-      if(writeCheckTarget_ != socket) {
+      if(*writeCheckTarget_ != *socket) {
         e_->deleteSocketForWriteCheck(writeCheckTarget_, this);
         e_->addSocketForWriteCheck(socket, this);
         writeCheckTarget_ = socket;
@@ -196,7 +190,7 @@ void PeerAbstractCommand::setNoCheck(bool check)
 
 void PeerAbstractCommand::updateKeepAlive()
 {
-  checkPoint_ = global::wallclock;
+  checkPoint_ = global::wallclock();
 }
 
 void PeerAbstractCommand::createSocket()
@@ -204,4 +198,9 @@ void PeerAbstractCommand::createSocket()
   socket_.reset(new SocketCore());
 }
 
+void PeerAbstractCommand::addCommandSelf()
+{
+  e_->addCommand(std::unique_ptr<Command>(this));
+}
+
 } // namespace aria2
